----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    10:01:22 10/08/2013 
-- Design Name: 
-- Module Name:    FIFO72x8192 - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;
Library UNIMACRO;
use UNIMACRO.vcomponents.all;

entity FIFO67x4096 is
		generic(ALMOSTFULL_OFFSET : bit_vector(15 downto 0) := x"0080";ALMOSTEMPTY_OFFSET : bit_vector(15 downto 0) := x"0080");
    Port ( wclk : in  STD_LOGIC;
           rclk : in  STD_LOGIC;
           fifo_rst : in  STD_LOGIC;
           fifo_en : in  STD_LOGIC;
           di : in  STD_LOGIC_VECTOR (66 downto 0);
           we : in  STD_LOGIC;
           re : in  STD_LOGIC;
           do : out  STD_LOGIC_VECTOR (66 downto 0);
           full : out  STD_LOGIC;
           empty : out  STD_LOGIC;
           data_avl : out  STD_LOGIC);
end FIFO67x4096;

architecture Behavioral of FIFO67x4096 is
signal fifo_re : std_logic := '0';
signal fifo_we : std_logic := '0';
signal fifo_Almostempty : std_logic := '0';
signal fifo_do: std_logic_vector(66 downto 0);
signal fifo_empty: std_logic_vector(7 downto 0);
signal do_vld: std_logic := '1';
type array8X12 is array (0 to 7) of std_logic_vector(11 downto 0);
signal rdcount : array8X12 := (others => (others => '0'));
signal wrcount : array8X12 := (others => (others => '0'));

begin
g_FIFO: for i in 0 to 6 generate
   i_FIFO : FIFO_DUALCLOCK_MACRO
   generic map (
      DEVICE => "7SERIES",            -- Target Device: "VIRTEX5", "VIRTEX6", "7SERIES" 
      ALMOST_FULL_OFFSET => ALMOSTFULL_OFFSET,  -- Sets almost full threshold
      ALMOST_EMPTY_OFFSET => ALMOSTEMPTY_OFFSET, -- Sets the almost empty threshold
      DATA_WIDTH => 9,   -- Valid values are 1-72 (37-72 only valid when FIFO_SIZE="36Kb")
      FIFO_SIZE => "36Kb",            -- Target BRAM, "18Kb" or "36Kb" 
      FIRST_WORD_FALL_THROUGH => TRUE) -- Sets the FIFO FWFT to TRUE or FALSE
   port map (
      ALMOSTEMPTY => open,   -- 1-bit output almost empty
      ALMOSTFULL => open,     -- 1-bit output almost full
      DO => fifo_do(i*9+8 downto i*9),                     -- Output data, width defined by DATA_WIDTH parameter
      EMPTY => fifo_empty(i),               -- 1-bit output empty
      FULL => open,                 -- 1-bit output full
      RDCOUNT => rdcount(i),           -- Output read count, width determined by FIFO depth
      RDERR => open,               -- 1-bit output read error
      WRCOUNT => wrcount(i),           -- Output write count, width determined by FIFO depth
      WRERR => open,               -- 1-bit output write error
      DI => di(i*9+8 downto i*9),                     -- Input data, width defined by DATA_WIDTH parameter
      RDCLK => rclk,               -- 1-bit input read clock
      RDEN => FIFO_re,                 -- 1-bit input read enable
      RST => fifo_rst,                   -- 1-bit input reset
      WRCLK => wclk,               -- 1-bit input write clock
      WREN => FIFO_we                  -- 1-bit input write enable
   );
end generate;
i_FIFO_MSB : FIFO_DUALCLOCK_MACRO
   generic map (
      DEVICE => "7SERIES",            -- Target Device: "VIRTEX5", "VIRTEX6", "7SERIES" 
      ALMOST_FULL_OFFSET => ALMOSTFULL_OFFSET,  -- Sets almost full threshold
      ALMOST_EMPTY_OFFSET => ALMOSTEMPTY_OFFSET, -- Sets the almost empty threshold
      DATA_WIDTH => 4,   -- Valid values are 1-72 (37-72 only valid when FIFO_SIZE="36Kb")
      FIFO_SIZE => "18Kb",            -- Target BRAM, "18Kb" or "36Kb" 
      FIRST_WORD_FALL_THROUGH => TRUE) -- Sets the FIFO FWFT to TRUE or FALSE
   port map (
      ALMOSTEMPTY => fifo_Almostempty,   -- 1-bit output almost empty
      ALMOSTFULL => full,     -- 1-bit output almost full
      DO => fifo_do(66 downto 63),                     -- Output data, width defined by DATA_WIDTH parameter
      EMPTY => fifo_empty(7),               -- 1-bit output empty
      FULL => open,                 -- 1-bit output full
      RDCOUNT => rdcount(7),           -- Output read count, width determined by FIFO depth
      RDERR => open,               -- 1-bit output read error
      WRCOUNT => wrcount(7),           -- Output write count, width determined by FIFO depth
      WRERR => open,               -- 1-bit output write error
      DI => di(66 downto 63),                     -- Input data, width defined by DATA_WIDTH parameter
      RDCLK => rclk,               -- 1-bit input read clock
      RDEN => FIFO_re,                 -- 1-bit input read enable
      RST => fifo_rst,                   -- 1-bit input reset
      WRCLK => wclk,               -- 1-bit input write clock
      WREN => FIFO_we                  -- 1-bit input write enable
   );
FIFO_we <= FIFO_en and we;
FIFO_re <= '1' when FIFO_en = '1' and fifo_empty = x"00" and (re = '1' or do_vld = '0') else '0';
data_avl <= not fifo_Almostempty;
empty <= not do_vld;
process(rclk)
begin
	if(rclk'event and rclk = '1')then
		if(re = '1' or do_vld = '0')then
			do <= fifo_do;
		end if;
	end if;
end process;
process(rclk, fifo_en)
begin
	if(fifo_en = '0')then
		do_vld <= '0';
	elsif(rclk'event and rclk = '1')then
		if(fifo_empty = x"00")then
			do_vld <= '1';
		elsif(re = '1')then
			do_vld <= '0';
		end if;
	end if;
end process;
end Behavioral;

